#!/usr/bin/env python

"""Some functions to work with divisors"""


__all__ = (
   'divisors',
   'prime_divisors',
   'is_prime',
   'next_prime',
   'primes',
)

_SMALL_PRIMES = [2, 3, 5, 7, 11, 13, 17, 19]


def divisors(N):
    """Iterates over all divisors of N.

    >>> tuple(divisors(2009))
    (7, 41, 49, 287)

    """
    for i in range(2, N):
        if N % i == 0:
            yield i
    

def is_prime(N):
    """Returns True if N is prime.

    >>> is_prime(2)
    True
    >>> is_prime(9)
    False

    """
    if N <= _SMALL_PRIMES[-1]:
        return N in _SMALL_PRIMES
    else:
        for i in range(2, N):
            if N % i == 0:
                return False
        else:
            return True


def prime_divisors(N):
    """Iterates over all prime divisors of N.

    >>> tuple(prime_divisors(2009))
    (7, 41)

    """
    for i in divisors(N):
        if is_prime(i):
            yield i
    

def next_prime(N):
    """Return the first prime > N

    >>> next_prime(5)
    7
    >>> next_prime(8)
    11

    """
    p = N + 1
    while not is_prime(p):
        p += 1
    return p


def primes():
    """Iterates over all primes

    >>> for i in primes():
    ...     if i >= 10:
    ...         break
    ...     print i
    2
    3
    5
    7
    >>>

    """
    p = 2
    while True:
        yield p
        p = next_prime(p)


if __name__ == "__main__":
    import doctest
    doctest.testmod()



